/*
 * This software is in the public domain under CC0 1.0 Universal plus a 
 * Grant of Patent License.
 * 
 * To the extent possible under law, the author(s) have dedicated all
 * copyright and related and neighboring rights to this software to the
 * public domain worldwide. This software is distributed without any
 * warranty.
 * 
 * You should have received a copy of the CC0 Public Domain Dedication
 * along with this software (see the LICENSE.md file). If not, see
 * <http://creativecommons.org/publicdomain/zero/1.0/>.
 */
package org.moqui.impl.screen

import groovy.transform.CompileStatic
import org.slf4j.LoggerFactory
import org.slf4j.Logger
import org.moqui.impl.util.FtlNodeWrapper
import org.moqui.context.ContextStack

@CompileStatic
class ScreenWidgets {
    protected final static Logger logger = LoggerFactory.getLogger(ScreenWidgets.class)

    protected Node widgetsNode
    protected FtlNodeWrapper widgetsFtlNode
    protected String location

    ScreenWidgets(Node widgetsNode, String location) {
        this.widgetsNode = widgetsNode
        this.location = location
        this.widgetsFtlNode = FtlNodeWrapper.wrapNode(widgetsNode)
    }

    void render(ScreenRenderImpl sri) {
        ContextStack cs = (ContextStack) sri.ec.context
        try {
            cs.push()
            cs.sri = sri
            cs.widgetsNode = widgetsFtlNode
            sri.template.createProcessingEnvironment(cs, sri.writer).process()
        } finally {
            cs.pop()
        }
    }
}
